package dicewars.test;


import dicewars.model.Country;
import dicewars.model.DiceWars;
import dicewars.model.Hex;
import dicewars.model.Player;
import dicewars.file.FileService;
import dicewars.util.Directions;
import dicewars.util.Point;
import javafx.scene.paint.Color;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;

import static java.lang.Math.sqrt;
import static java.lang.System.out;
import static java.lang.System.runFinalization;

/*
 *  This is where you should test methods
 *  Add own tests if needed or extend existing
 */
public class Test {

    public static void main(String[] args) {
        try {
            new Test().test();     // Because of file handling (exceptions)
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    // All tests
    private void test() throws IOException {
        // Test of Hex, see TestHex file
        // testFileService();
        //testDiceWarsSetUp();
        //testCountryNeighbours();
        //testCountryBorders();
    }

    private void testFileService() throws IOException {
        Map<Color, List<Hex>> regionsOut = new HashMap<>();
        List<Hex> hexes = generateHexGrid(5);
        // Generate some data
        regionsOut.put(Color.WHITE, hexes.subList(0, 10));
        regionsOut.put(Color.BLUE, hexes.subList(10, hexes.size()));
        // Write out data to file ...
        Path path = Paths.get("src/dicewars/test/junk.txt");
        FileService.writeHexGrid(path, regionsOut);
        // ... and read back same file
        Map<Color, List<Hex>> regionsIn = FileService.readHexGrid(path);

        // Check data read is the same as written ...
        List<Color> cs1 = new ArrayList<>();
        List<Hex> hs1 = new ArrayList<>();
        // Don't care about the ->, will be covered in later courses
        regionsIn.forEach((color, hs) -> {
            cs1.add(color);
            hs1.addAll(hs);
        });

        List<Color> cs2 = new ArrayList<>();
        List<Hex> hs2 = new ArrayList<>();
        regionsOut.forEach((color, hs) -> {
            cs2.add(color);
            hs2.addAll(hs);
        });

        out.println(cs1.containsAll(cs2));  // ... here
        out.println(hs1.containsAll(hs2));

    }


    private void testDiceWarsSetUp() {
        // Using hard coded test data : 4 Players, 13 Countries (no Hex'es in Countries)
        DiceWars dw = new DiceWars(getCountries(), getPlayers());
        List<Country> cs = dw.getCountries();
        out.println(cs.size() == 13);
        List<Player> ps = dw.getPlayers();
        out.println(ps.size() == 4);
        boolean b = true;
        for (Country c : cs) {
            b = b && c.getOwner() != null;  // All countries should have owner
        }
        out.println(b);
        for (Country c : cs) {
            b = b && c.getNDices() >= 1;  // All countries should have at least 1 dice
        }
        out.println(b);
        for (Player p : ps) {
            // TODO   Players have at least 3 countries
        }
        out.println(b);
        out.println(ps.contains(dw.getActual()));   // There should be an actual player
    }

    private void testCountryNeighbours() throws IOException {
        // Three slim countries
        List<Hex> hs = Arrays.asList(new Hex(0, 0), new Hex(1, 0), new Hex(2, 0));
        Country c1 = new Country(hs);
        hs = Arrays.asList(new Hex(0, 1), new Hex(1, 1), new Hex(2, 1));
        Country c2 = new Country(hs);
        hs = Arrays.asList(new Hex(0, 2), new Hex(1, 2), new Hex(2, 2));
        Country c3 = new Country(hs);

        // TODO

    }

    private void testCountryBorders() {
        // A small Country
        List<Hex> hs = Arrays.asList(new Hex(0, 0), new Hex(1, 0), new Hex(2, 0),
                new Hex(0, 1), new Hex(1, 1), new Hex(2, 1),
                new Hex(0, 2), new Hex(1, 2), new Hex(2, 2));
        Country c4 = new Country(hs);
        List<Directions> dirs = c4.getBorders(new Hex(0, 0));  // Upper left corner
        out.println(dirs.size() == 4);
        // NOTE: Directions flipped, see Directions
        out.println(dirs.containsAll(Arrays
                .asList(Directions.NE, Directions.E, Directions.SE, Directions.SW)));
        dirs = c4.getBorders(new Hex(1, 1));  // Center has no borders
        out.println(dirs.size() == 0);
    }

    // ---------- Helpers --------------------------

    private List<Hex> generateHexGrid(int side) {
        List<Hex> hexes = new ArrayList<>();
        for (int yh = 0; yh < side; yh++) {
            for (int xh = -yh / 2; xh < side - yh / 2; xh++) {
                hexes.add(new Hex(xh, yh));
            }
        }
        return hexes;
    }

    private List<Country> getCountries() {
        List<Country> sc = new ArrayList<>();
        for (int i = 0; i < 13; i++) {
            sc.add(new Country(new ArrayList<>())); // Don't need the hexes for testing DiceWars
        }
        return sc;
    }

    private List<Player> getPlayers() {
        return Arrays.asList(new Player("Pelle", Color.LIGHTGREEN), new Player("Olle", Color.LIGHTBLUE),
                new Player("Fia", Color.YELLOW), new Player("Lisa", Color.ORANGE));
    }


}
